@@ -1,58 +1,70 @@
-//! Defines serializing and deserializing traits and common record types.
-//!
-//! Any object scanned by readers or written by writers must implement
-//! [PcdDeserialize](crate::record::PcdDeserialize) or [PcdSerialize](crate::record::PcdSerialize)
-//! respectively.
-//!
-//! These traits are not intended to implemented manually.
-//! Please use derive macro instead. For example,
-//!
-//! ```rust
-//! use pcd_rs::{PcdDeserialize, PcdSerialize};
-//!
-//! #[derive(PcdDeserialize, PcdSerialize)]
-//! pub struct TimestampedPoint {
-//!     x: f32,
-//!     y: f32,
-//!     z: f32,
-//!     timestamp: u32,
-//! }
-//! ```
-//!
-//! The derive macro accepts normal structs and tuple structs, but does not accept unit structs.
-//!
-//! [PcdDeserialize](crate::record::PcdDeserialize) allows fields with either primitive type,
-//! array of primitive type or [Vec](<std::vec::Vec>) of primitive type.
-//!
-//! [PcdSerialize](crate::record::PcdSerialize) allows fields with either primitive type or
-//! array of primitive type. The [Vec](<std::vec::Vec>) is ruled out since the length
-//! is not determined in compile-time.
-//!
-//! Make sure struct field names match the `FIELDS` header in PCD data.
-//! Otherwise it panics at runtime. You can specify the exact name in header or bypass name check
-//! with attributes. The name check are automatically disabled for tuple structs.
-//!
-//! ```rust
-//! use pcd_rs::PcdDeserialize;
-//!
-//! #[derive(PcdDeserialize)]
-//! pub struct TimestampedPoint {
-//!     x: f32,
-//!     y: f32,
-//!     z: f32,
-//!     #[pcd_rename("true_name")]
-//!     rust_name: u32,
-//!     #[pcd_ignore_name]
-//!     whatever_name: u32,
-//! }
-//! ```
+#[doc = r##"
+Defines serializing and deserializing traits and common record types.
 
+Any object scanned by readers or written by writers must implement
+[PcdDeserialize](crate::record::PcdDeserialize) or [PcdSerialize](crate::record::PcdSerialize)
+respectively.
+
+These traits are not intended to implemented manually.
+Please use derive macro instead. For example,
+
+"##]
+#[cfg_attr(
+    feature = "derive",
+    doc = r##"
+```rust
+use pcd_rs::{PcdDeserialize, PcdSerialize};
+
+#[derive(PcdDeserialize, PcdSerialize)]
+pub struct TimestampedPoint {
+    x: f32,
+    y: f32,
+    z: f32,
+    timestamp: u32,
+}
+```
+"##
+)]
+#[doc = r##"
+The derive macro accepts normal structs and tuple structs, but does not accept unit structs.
+
+[PcdDeserialize](crate::record::PcdDeserialize) allows fields with either primitive type,
+array of primitive type or [Vec](<std::vec::Vec>) of primitive type.
+
+[PcdSerialize](crate::record::PcdSerialize) allows fields with either primitive type or
+array of primitive type. The [Vec](<std::vec::Vec>) is ruled out since the length
+is not determined in compile-time.
+
+Make sure struct field names match the `FIELDS` header in PCD data.
+Otherwise it panics at runtime. You can specify the exact name in header or bypass name check
+with attributes. The name check are automatically disabled for tuple structs.
+"##]
+#[cfg_attr(
+    feature = "derive",
+    doc = r##"
+```rust
+use pcd_rs::PcdDeserialize;
+
+#[derive(PcdDeserialize)]
+pub struct TimestampedPoint {
+    x: f32,
+    y: f32,
+    z: f32,
+    #[pcd(rename = "true_name")]
+    rust_name: u32,
+    #[pcd(ignore)]
+    whatever_name: u32,
+}
+```
+"##
+)]
 use crate::{
     error::Error,
     metas::{FieldDef, Schema, ValueKind},
 };
 use anyhow::{bail, Result};
 use byteorder::{LittleEndian, ReadBytesExt, WriteBytesExt};
+use num_traits::FromPrimitive;
 use std::io::prelude::*;
 
 /// [PcdDeserialize](crate::record::PcdDeserialize) is analogous to a _point_ returned from a reader.
@@ -165,6 +177,44 @@ impl DynRecord {
                 )
             })
     }
+
+    pub fn xyz<T>(self) -> Option<[T; 3]>
+    where
+        T: 'static + Copy + FromPrimitive,
+    {
+        let (x, y, z) = match &self.0[0..3] {
+            [Field::I8(x), Field::I8(y), Field::I8(z)] => {
+                (T::from_i8(x[0]), T::from_i8(y[0]), T::from_i8(z[0]))
+            }
+            [Field::I16(x), Field::I16(y), Field::I16(z)] => {
+                (T::from_i16(x[0]), T::from_i16(y[0]), T::from_i16(z[0]))
+            }
+            [Field::I32(x), Field::I32(y), Field::I32(z)] => {
+                (T::from_i32(x[0]), T::from_i32(y[0]), T::from_i32(z[0]))
+            }
+            [Field::U8(x), Field::U8(y), Field::U8(z)] => {
+                (T::from_u8(x[0]), T::from_u8(y[0]), T::from_u8(z[0]))
+            }
+            [Field::U16(x), Field::U16(y), Field::U16(z)] => {
+                (T::from_u16(x[0]), T::from_u16(y[0]), T::from_u16(z[0]))
+            }
+            [Field::U32(x), Field::U32(y), Field::U32(z)] => {
+                (T::from_u32(x[0]), T::from_u32(y[0]), T::from_u32(z[0]))
+            }
+            [Field::F32(x), Field::F32(y), Field::F32(z)] => {
+                (T::from_f32(x[0]), T::from_f32(y[0]), T::from_f32(z[0]))
+            }
+            [Field::F64(x), Field::F64(y), Field::F64(z)] => {
+                (T::from_f64(x[0]), T::from_f64(y[0]), T::from_f64(z[0]))
+            }
+            _ => panic!("Point info {:?} is not complete!", &self.0),
+        };
+        if let (Some(x), Some(y), Some(z)) = (x, y, z) {
+            Some([x, y, z])
+        } else {
+            None
+        }
+    }
 }
 
 impl PcdSerialize for DynRecord {
